CGListContainer_mt = {__index = CGListContainer}

-- Constructor
function CGListContainer:create(t, m, r)
	local new_inst = {titles = t, model = m, rows = r, offset = 0, selected = {}, title_labels = {}, row_labels = {}}
	if (r < 4) then new_inst.rows = 4 end
	
	local f = love.graphics.getFont()
	new_inst.width = 20
	for _,v in ipairs(t) do
		new_inst.width = new_inst.width + f:getWidth(v) + 20
	end
	new_inst.height = new_inst.rows*12
	
	local obj = CGPanel:create()
	obj.bgColor={r=200, g=200, b=200}
	obj.width, obj.height = new_inst.width, new_inst.height
	obj.onMouseDown = function(x,y,button)
		new_inst:setOffset(new_inst.scrollbar.value)
	end
	new_inst.panel = obj
	
	obj = CGScrollbar:create(new_inst.height)
	obj.posX = new_inst.width - 20
	obj:setMaxValue(math.max(# m - r + 2,0))
	obj.onScrollDown = function(x,y,button)
		new_inst:setOffset(new_inst.scrollbar.value)
	end
	obj.onScrollUp = obj.onScrollDown
	new_inst.scrollbar = obj
	new_inst.panel:addWidget(obj)
	
	
	local dx = 0
	local dy = 0
	obj = {}
	for i,v in ipairs(t) do
		table.insert(obj,CGLabel:create(v))
		obj[i].posX = dx
		dx = dx + 20 + f:getWidth(v)
		new_inst.panel:addWidget(obj[i])
	end
	
	self.title_labels = obj
	
	for i = 1,r do
		print(dy)
		obj = {}
		dx = 0
		for i,v in ipairs(t) do
			table.insert(obj,CGLabel:create(""))
			obj[i].posX = dx
			obj[i].posY = dy
			dx = dx + 20 + f:getWidth(v)
			new_inst.panel:addWidget(obj[i])
		end
		dy = dy + 12		
		table.insert(new_inst.row_labels, obj)
	end
	
	setmetatable(new_inst, CGListContainer_mt)
	return new_inst
end

function CGListContainer:setOffset(o)
	self.offset = o
	if (o > self.scrollbar.maxValue) then self.offset = self.scrollbar.maxValue end
	for i = 1, self.rows-1 do
		for t = 1, # self.titles do
			self.row_labels[i+1][t].text = (self.model[i+self.offset] and self.model[i+self.offset][t]) or ""
		end
	end
end

function CGListContainer:addRow(r)
	table.insert(self.model, r)
	self.scrollbar:setMaxValue(# self.model - self.rows + 1)
	self:setOffset(self.offset)
end

function CGListContainer:_onMouseDown(x,y,button)
	self.panel:_onMouseDown(x,y,button)
	if (self.onMouseDown) then self.onMouseDown(x,y,button) end
end

function CGListContainer:_onMouseUp(x,y,button)
	self.panel:_onMouseUp(x,y,button)
	if (self.onMouseUp) then self.onMouseUp(x,y,button) end
end

function CGListContainer:_onMouseOver(x,y,d)
	self.panel:_onMouseOver(x,y,d)
	if (self.onMouseOver) then self.onMouseOver(x,y,d) end
end

function CGListContainer:_onMouseNotOver(x,y,d)
	self.panel:_onMouseNotOver(x,y,d)
	if (self.onMouseNotOver) then self.onMouseNotOver(x,y,d) end
end

function CGListContainer:_onScrollUp(x,y,button)
	self.scrollbar:_onScrollUp(x,y,button)
	if (self.onScrollUp) then self.onScrollUp(x,y,button) end
end

function CGListContainer:_onScrollDown(x,y,button)
	self.scrollbar:_onScrollDown()
	if (self.onScrollDown) then self.onScrollDown(x,y,button) end
end

function CGListContainer:draw()
	love.graphics.push()
	love.graphics.translate(self.posX, self.posY)
	self.panel:draw()
	love.graphics.pop()
end